
#ifndef __CLIENT_NET_RELAY_TYPES_H_
#define __CLIENT_NET_RELAY_TYPES_H_

#if (defined(WIN32) || defined(WIN64))
#define MSG_MAGIC (0x20141104)
#include <WinSock2.h>
#else
constexpr unsigned int MSG_MAGIC(0x20141104);
#include <arpa/inet.h>
#endif

enum MSG_TYPE : unsigned short
{
    MSG_TYPE_OTHERS = 0, 
    MSG_TYPE_FRAME_SN = 1,
    MSG_TYPE_FRAME_I = 2,
    MSG_TYPE_FRAME_P = 3,
    MSG_TYPE_FRAME_A = 4,
    MSG_TYPE_ERRCODE = 5,
    MSG_TYPE_FRAME_VOICE_SN = 6,
    MSG_TYPE_SPEED_TEST_SN = 7,
    MSG_TYPE_FRAME_B = 8,
    MSG_TYPE_CONTROL = 9, ///< use it to close the stream
    MSG_TYPE_FRAME_AAC = 10,
    MSG_TYPE_FRAME_OPUS = 12, 
    MSG_TYPE_FRAME_HEVC_I = 13,
    MSG_TYPE_FRAME_HEVC_P = 14,
    MSG_TYPE_FRAME_HEVC_B = 15, 
    MSG_TYPE_MULTI = 200 ///< ·
};

// · op
#define P_R_MC_ENTER 8101 ///< Ϊα
#define O_R_MC_ENTER  8102 ///< ĳһڳΪα
#define P_R_MC_LEAVE 8103 ///< α뿪
#define P_R_MC_INFOS 8104 ///< αϢ
#define O_R_MC_SET_STATUS  8105 ///< ĳһα״̬
#define O_R_MC_SET_NUMBER 8106 ///< üα()
#define O_R_MC_SET_PREVIEW 8107 ///< Ԥ
#define O_R_MC_LEAVE 8108 ///< ߳α
#define R_P_MC_ENTER 8201 ///< ֪ͨΪα
#define R_P_MC_INFOS 8204 ///< ؼαϢ(ȫ)
#define R_P_MC_SET_PREVIEW 8207 ///< ֪ͨ relay Ԥ
#define R_P_MC_ENTER_NOTIFY 8301 ///< ֪ͨα
#define R_P_MC_LEAVE_NOTIFY 8302 ///< ֪ͨα뿪
#define R_P_MC_NUMBER_NOTIFY  8303 ///< ֪ͨüα

enum StreamType : unsigned short
{
    STREAM_TYPE_UNKNOWN = 0, 
    STREAM_TYPE_VIDEO = 1,
    STREAM_TYPE_AUDIO = 2,
    STREAM_TYPE_OTHER = 3,
    STREAM_TYPE_CONTROL_CLOSE = 4,
    STREAM_TYPE_VIDEO_HEVC = 5,
};

enum EncryptType : unsigned short
{
    ENCRYPT_TYPE_NONE = 0,
    ENCRYPT_TYPE_CHACHA2 = 1,
};

enum TLVTypeWithServer : unsigned short
{
    X_TLV_TYPE_AUTH_SN = 1,
    X_TLV_TYPE_AUTH_ENCRYPT = 2,
    X_TLV_TYPE_ERRCODE = 3,
    X_TLV_TYPE_NEED_RECONNECT = 4,
    X_TLV_TYPE_CLUSTER_ID = 5,
    X_TLV_TYPE_NO_AUTO_CLOSE = 6,
    X_TLV_TYPE_SOURCE_TYPE = 7, ///< 1000 means camera, 1001 means huajiao
    X_TLV_TYPE_UPLOAD_KEY = 10, ///< 1 means publish, 0 means subscribe
    X_TLV_TYPE_APP_NAME = 11, ///< 1 means publish, 0 means subscribe
    X_TLV_TYPE_RESP_TESTING = 16, 
    X_TLV_TYPE_RESP_TESTING_SIZE = 17, 
    X_TLV_REQUEST_FRAME_TYPE = 21, 
    // ·TLV
    X_TLV_TYPE_MULTI_USER_NAME = 30,
    X_TLV_TYPE_MULTI_USER_ID,
    X_TLV_TYPE_MULTI_GUSTNUM,
    X_TLV_TYPE_MULTI_STATUS,
    X_TLV_TYPE_MULTI_ARRAY_LEN, 
    X_TLV_TYPE_MULTI_VERSION,
};

#pragma pack(push, 1)
struct msg_head
{
    unsigned int magic; ///< magic is 0x20141104(BE)
    unsigned short type;
    unsigned int size;
    unsigned int get_size() const {
        return ntohl(size);
    }
    void set_size(unsigned int _size) {
        size = htonl(_size);
    }
    unsigned short get_type() const {
        return ntohs(type);
    }
    void set_type(unsigned short _type) {
        type = htons(_type);
    }
};

struct data_header
{
    unsigned long long serial;
    unsigned long long timestamp;
    unsigned int frametype;
    unsigned int iskey;
    unsigned int encrypt_type;
    unsigned short width;
	unsigned short height;
    unsigned int lan_ip;
	unsigned short lan_port;
    unsigned char quality;
    unsigned char reserved1;
    unsigned int reserved2;
};

// the old tlv(type length value) struct
struct tlv_header
{
    unsigned short type;
    unsigned short length;
    unsigned short get_type() const {
        return ntohs(type);
    }
    unsigned short get_length() const {
        return ntohs(length);
    }
    void set_type(unsigned short _type) {
        type = htons(_type);
    }
    void set_length(unsigned short _length) {
        length = htons(_length);
    }
};

typedef struct
{
    unsigned short op; ///< 
    unsigned short fromId; ///<  ID
    unsigned short toId; ///<  ID
    unsigned short version; ///< 汾
    unsigned short reserved1; ///< Ԥֶ
    unsigned short reserved2; ///< Ԥֶ
} multi_head;
#pragma pack(pop)

#ifndef __APPLE__
#if (defined(WIN32) || defined(WIN64))
#else
static int is_little_endian()
{
    int x = 1;
    return *(char*)&x;
}

inline unsigned long long ntohll(unsigned long long val)
{
    if (is_little_endian()) {
        return (((unsigned long long )htonl((int)((val << 32) >> 32))) << 32) | (unsigned int)htonl((int)(val >> 32));
    }
    else {
        return val;
    }
}

inline unsigned long long htonll(unsigned long long val)
{
    if (is_little_endian()) {
        return (((unsigned long long)htonl((int)((val << 32) >> 32))) << 32) | (unsigned int)htonl((int)(val >> 32));
    }
    else {
        return val;
    }
}
#endif
#endif

#endif

